def select_by_kp(image_integral, anchors):
    pass


def nms(dets, thresh):
    pass


def box_connect(imageIntegral, anchors_in, h_gap, width, height):
    pass


from .select_by_kp import select_by_kp, nms, box_connect
